@@ -20,7 +20,7 @@
   
 /*==============================================================================
 
-  $Id: drv_AF.c,v 1.21 1999/03/24 06:00:05 miod Exp $
+  $Id: drv_AF.c,v 1.25 1999/06/28 03:00:05 miod Exp $
 
   Driver for output on AF audio server.
 
@@ -28,7 +28,7 @@
 
 /*
 
-	Written by Roine Gustafsson <e93_rog@e.kth.se> Oct 25, 1995
+	Written by Roine Gustafsson <e93_rog@e.kth.se>
   
 	Portability:
 	Unixes running Digital AudioFile library, available from
@@ -65,11 +65,26 @@ static	AFAudioConn *AFaud=NULL;
 static	ATime AFtime;
 static	AC AFac;
 static	AFDeviceDescriptor *AFdesc;
+static	CHAR *soundbox=NULL;
+
+static void AF_CommandLine(CHAR *cmdline)
+{
+	CHAR *machine=MD_GetAtom("machine",cmdline,0);
+
+	if(machine) {
+		if(soundbox) free(soundbox);
+		soundbox=machine;
+	}
+}
 
 static BOOL AF_IsThere(void)
 {
-	/* I'll think of a detection routine ... sometime */
-	return 1;
+	if ((AFaud=AFOpenAudioConn(soundbox)))
+		AFCloseAudioConn(AFaud);
+		AFaud=NULL;
+		return 1;
+	} else
+		return 0;
 }
 
 static BOOL AF_Init(void)
@@ -80,8 +95,7 @@ static BOOL AF_Init(void)
 	ADevice device;
 	int n;
   
-	AFaud=AFOpenAudioConn("");
-	if (!AFaud) {
+	if (!(AFaud=AFOpenAudioConn(soundbox))) {
 		_mm_errno=MMERR_OPENING_AUDIO;
 		return 1;
 	}
@@ -149,12 +163,6 @@ static void AF_Exit(void)
 	}
 }
 
-static BOOL AF_Reset(void)
-{
-	AF_Exit();
-	return AF_Init();
-}
-
 static void AF_Update(void)
 {
 	ULONG done;
@@ -176,6 +184,9 @@ MDRIVER drv_AF={
 	"AF driver",
 	"AudioFile driver v1.3",
 	0,255,
+	"audiofile",
+
+	AF_CommandLine,	
 	AF_IsThere,
 	VC_SampleLoad,
 	VC_SampleUnload,
@@ -183,7 +194,7 @@ MDRIVER drv_AF={
 	VC_SampleLength,
 	AF_Init,
 	AF_Exit,
-	AF_Reset,
+	NULL,
 	VC_SetNumVoices,
 	AF_PlayStart,
 	VC_PlayStop,
