@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_dart.c,v 1.15 1999/03/24 06:00:05 miod Exp $
+  $Id: drv_dart.c,v 1.19 1999/06/28 03:00:05 miod Exp $
 
   Driver for output on OS/2 MMPM/2 using direct audio (DART)
 
@@ -240,19 +240,15 @@ static void OS2_Dart_PlayStop(void)
 	VC_PlayStop();
 }
 
-static BOOL OS2_Dart_Reset(void)
-{
-	OS2_Dart_Exit();
-	return OS2_Dart_Init();
-}
-
 MDRIVER drv_dart=
 {
 	NULL,
 	"Direct audio (DART)",
 	"OS/2 DART driver v1.1",
-	0,
-	255,
+	0,255,
+	"dart",
+
+	NULL,
 	OS2_Dart_IsPresent,
 	VC_SampleLoad,
 	VC_SampleUnload,
@@ -260,7 +256,7 @@ MDRIVER drv_dart=
 	VC_SampleLength,
 	OS2_Dart_Init,
 	OS2_Dart_Exit,
-	OS2_Dart_Reset,
+	NULL,
 	VC_SetNumVoices,
 	OS2_Dart_PlayStart,
 	OS2_Dart_PlayStop,
