@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_esd.c,v 1.21 1999/03/24 06:00:05 miod Exp $
+  $Id: drv_esd.c,v 1.25 1999/06/28 03:00:05 miod Exp $
 
   Driver for the Enlightened sound daemon (EsounD)
 
@@ -58,7 +58,7 @@
 #ifdef MIKMOD_DYNAMIC
 /* runtime link with libesd */
 /* note that since we only need the network part of EsounD, we don't need to
-   load libasound.so as well */
+   load libasound.so or libaudiofile.so as well */
 #ifdef HAVE_ESD_CLOSE
 static int(*esd_closestream)(int);
 #else
@@ -78,6 +78,7 @@ static void* libesd=NULL;
 static	int sndfd=-1;
 static	esd_format_t format;
 static	SBYTE *audiobuffer=NULL;
+static	CHAR *espeaker=NULL;
 
 /* Backwards compatibility stuff */
 
@@ -102,17 +103,17 @@ int writes;
 
 static BOOL ESD_Link(void)
 {
-	if(libesd) return 0;
+	if (libesd) return 0;
 
 	/* load libesd.so */
 	libesd=dlopen("libesd.so.0",RTLD_LAZY|RTLD_GLOBAL);
-	if(!libesd) return 1;
+	if (!libesd) return 1;
 
 	/* resolve function references */
 #ifdef HAVE_ESD_CLOSE
-	if(!(esd_closestream=dlsym(libesd,"esd_close"))) return 1;
+	if (!(esd_closestream=dlsym(libesd,"esd_close"))) return 1;
 #endif
-	if(!(esd_playstream=dlsym(libesd,"esd_play_stream"))) return 1;
+	if (!(esd_playstream=dlsym(libesd,"esd_play_stream"))) return 1;
 
 	return 0;
 }
@@ -124,7 +125,7 @@ static void ESD_Unlink(void)
 #endif
 	esd_playstream=NULL;
 
-	if(libesd) {
+	if (libesd) {
 		dlclose(libesd);
 		libesd=NULL;
 	}
@@ -136,13 +137,13 @@ static ssize_t esd_writebuf(int fd,const void *buffer,size_t count)
 {
 	ssize_t start=0;
 
-	while(start<count) {
+	while (start<count) {
 		ssize_t res;
 
 		res=write(fd,(char*)buffer+start,count-start);
-		if(res<0) {
+		if (res<0) {
 			/* connection lost */
-			if(errno==EPIPE)
+			if (errno==EPIPE)
 				return -1-start;
 		} else
 			start+=res;
@@ -150,6 +151,16 @@ static ssize_t esd_writebuf(int fd,const void *buffer,size_t count)
 	return start;
 }
 
+static void ESD_CommandLine(CHAR *cmdline)
+{
+	CHAR *ptr=MD_GetAtom("machine",cmdline,0);
+
+	if (ptr) {
+		if (espeaker) free(espeaker);
+		espeaker=ptr;
+	}
+}
+
 static BOOL ESD_IsThere(void)
 {
 	int fd,retval;
@@ -157,13 +168,18 @@ static BOOL ESD_IsThere(void)
 #ifdef MIKMOD_DYNAMIC
 	if (ESD_Link()) return 0;
 #endif
-	/* Try to esablish a connection with default esd settings */
-	if((fd=esd_playstream(ESD_BITS16|ESD_STEREO|ESD_STREAM|ESD_PLAY,
-	                       ESD_DEFAULT_RATE,NULL,"libmikmod"))<0)
+	/* Try to esablish a connection with default esd settings, but we don't
+	   want esdlib to spawn esd if esd is not running ! */
+	if (setenv("ESD_NO_SPAWN","1",0))
 		retval=0;
 	else {
-		esd_closestream(fd);
-		retval=1;
+		if ((fd=esd_playstream(ESD_BITS16|ESD_STEREO|ESD_STREAM|ESD_PLAY,
+		                       ESD_DEFAULT_RATE,espeaker,"libmikmod"))<0)
+			retval=0;
+		else {
+			esd_closestream(fd);
+			retval=1;
+		}
 	}
 #ifdef MIKMOD_DYNAMIC
 	ESD_Unlink();
@@ -177,12 +193,17 @@ static BOOL ESD_Init_internal(void)
 	       (md_mode&DMODE_STEREO?ESD_STEREO:ESD_MONO)|ESD_STREAM|ESD_PLAY;
 
 	/* make sure we can open an esd stream with our parameters */
-	if((sndfd=esd_playstream(format,md_mixfreq,NULL,"libmikmod"))<0) {
-		_mm_errno=MMERR_OPENING_AUDIO;
+	if (!(setenv("ESD_NO_SPAWN","1",0))) {
+		if ((sndfd=esd_playstream(format,md_mixfreq,espeaker,"libmikmod"))<0) {
+			_mm_errno=MMERR_OPENING_AUDIO;
+			return 1;
+		}
+	} else {
+		_mm_errno=MMERR_OUT_OF_MEMORY;
 		return 1;
 	}
 
-	if(!(audiobuffer=(SBYTE*)_mm_malloc(ESD_BUF_SIZE*sizeof(char))))
+	if (!(audiobuffer=(SBYTE*)_mm_malloc(ESD_BUF_SIZE*sizeof(char))))
 		return 1;
 
 	return VC_Init();
@@ -209,6 +230,7 @@ static void ESD_Exit_internal(void)
 	if (sndfd>=0) {
 		esd_closestream(sndfd);
 		sndfd=-1;
+		signal(SIGPIPE,SIG_DFL);
 	}
 }
 
@@ -224,12 +246,13 @@ static void ESD_Update_internal(int count)
 {
 	static time_t losttime;
 
-	if(sndfd>=0) {
-		if(esd_writebuf(sndfd,audiobuffer,count)<0) {
+	if (sndfd>=0) {
+		if (esd_writebuf(sndfd,audiobuffer,count)<0) {
 			/* if we lost our connection with esd, clean up and work as the
 			   nosound driver until we can reconnect */
 			esd_closestream(sndfd);
 			sndfd=-1;
+			signal(SIGPIPE,SIG_DFL);
 			losttime=time(NULL);
 		}
 	} else {
@@ -238,10 +261,11 @@ static void ESD_Update_internal(int count)
 		if (time(NULL)-losttime>=5) {
 			losttime=time(NULL);
 			/* Attempt to reconnect every 5 seconds */
-			if((sndfd=esd_playstream(format,md_mixfreq,NULL,"libmikmod"))>=0) {
-				VC_SilenceBytes(audiobuffer,ESD_BUF_SIZE);
-				esd_writebuf(sndfd,audiobuffer,ESD_BUF_SIZE);
-			}
+			if (!(setenv("ESD_NO_SPAWN","1",0)))
+				if ((sndfd=esd_playstream(format,md_mixfreq,espeaker,"libmikmod"))>=0) {
+					VC_SilenceBytes(audiobuffer,ESD_BUF_SIZE);
+					esd_writebuf(sndfd,audiobuffer,ESD_BUF_SIZE);
+				}
 		}
 	}
 }
@@ -258,13 +282,16 @@ static void ESD_Pause(void)
 
 static BOOL ESD_PlayStart(void)
 {
-	if(sndfd<0)
-		if((sndfd=esd_playstream(format,md_mixfreq,NULL,"libmikmod"))<0) {
-			_mm_errno=MMERR_OPENING_AUDIO;
-			return 1;
-		}
+	if (sndfd<0)
+		if (!(setenv("ESD_NO_SPAWN","1",0)))
+			if ((sndfd=esd_playstream(format,md_mixfreq,espeaker,"libmikmod"))<0) {
+				_mm_errno=MMERR_OPENING_AUDIO;
+				return 1;
+			}
 	/* since the default behaviour of SIGPIPE on most Unices is to kill the
-	   program, we'll prefer handle EPIPE ourselves should the esd die */
+	   program, we'll prefer handle EPIPE ourselves should the esd die - recent
+	   esdlib use a do-nothing handler, which prevents us from receiving EPIPE,
+	   so we override this */
 	signal(SIGPIPE,SIG_IGN);
 
 	/* silence buffers */
@@ -289,16 +316,19 @@ static void ESD_PlayStop(void)
 
 static BOOL ESD_Reset(void)
 {
-	ESD_Exit();
-	return ESD_Init();
+	ESD_Exit_internal();
+	return ESD_Init_internal();
 }
 
 MDRIVER drv_esd={
 	NULL,
 	"Enlightened sound daemon",
 	/* use the same version number as the EsounD release it works with */
-	"Enlightened sound daemon (EsounD) driver v0.2.8",
+	"Enlightened sound daemon (EsounD) driver v0.2.12",
 	0,255,
+	"esd",
+
+	ESD_CommandLine,
 	ESD_IsThere,
 	VC_SampleLoad,
 	VC_SampleUnload,
