@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_ultra.c,v 1.4 1999/03/24 06:00:05 miod Exp $
+  $Id: drv_ultra.c,v 1.9 1999/06/28 03:00:05 miod Exp $
 
   Driver for the Linux Ultrasound driver
 
@@ -28,9 +28,10 @@
 
 /*
 
-	Written by Andy Lo A Foe <arloafoe@cs.vu.nl>
+	Written by Andy Lo A Foe <andy@alsa-project.org>
+
 	Updated to work with later versions of both the ultrasound driver and
-	libmikmod by C. Ray C. <thoughtm@swivel.aimhi.com>
+	libmikmod by C. Ray C. <crayc@pyro.net>
 
 */
 
@@ -415,6 +416,8 @@ static BOOL Ultra_Init_internal(void)
 		return 1;
 	}
 
+	md_mode&=~(DMODE_SOFT_MUSIC|DMODE_SOFT_SNDFX);
+
 	/* Open the GUS card */
 	ultra_dev=getenv("MM_ULTRA")?atoi(getenv("MM_ULTRA")):0;
 	if ((gf1flag=libgus_open(ultra_dev,SIZE_OF_SEQBUF,0))<0) {
@@ -714,6 +717,9 @@ MDRIVER drv_ultra={
 	"Ultrasound driver",
 	"Linux Ultrasound driver v1.11",
 	GUS_CHANNELS,0,
+	"ultra",
+
+	NULL,
 	Ultra_IsThere,
 	Ultra_SampleLoad,
 	Ultra_SampleUnload,
