@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mikmod_internals.h,v 1.16 1999/03/24 06:00:05 miod Exp $
+  $Id: mikmod_internals.h,v 1.21 1999/06/28 03:00:05 miod Exp $
 
   MikMod sound library internal definitions
 
@@ -29,10 +29,17 @@
 #ifndef _MIKMOD_INTERNALS_H
 #define _MIKMOD_INTERNALS_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifdef HAVE_MALLOC_H
 #include <malloc.h>
 #endif
 #include <stdarg.h>
+#if defined(__OS2__)||defined(__EMX__)||defined(WIN32)
+#define strcasecmp(s,t) stricmp(s,t)
+#endif
 
 #include <mikmod_build.h>
 
@@ -42,15 +49,16 @@
 
 /*========== More type definitions */
 
+/* SLONGLONG : 64bit, signed */
 #if defined(__alpha)
-typedef long		SLONGLONG;	/* 8 bytes, signed */
+typedef long		SLONGLONG;
 #define NATIVE_64BIT_INT
 #elif defined(__WATCOMC__)
-typedef __int64		SLONGLONG;  /* 8 bytes, signed */
-#elif defined(WIN32)
-typedef LONGLONG	SLONGLONG;	/* 8 bytes, signed */
+typedef __int64		SLONGLONG;
+#elif defined(WIN32) && !defined(__MWERKS__)
+typedef LONGLONG	SLONGLONG;
 #else
-typedef long long	SLONGLONG;  /* 8 bytes, signed */
+typedef long long	SLONGLONG;
 #endif
 
 /*========== Error handling */
@@ -67,6 +75,9 @@ extern void* _mm_calloc(size_t,size_t);
 /*========== MT stuff */
 
 #ifdef HAVE_PTHREAD
+#if defined(__OpenBSD__) && !defined(_POSIX_THREADS)
+#define _POSIX_THREADS
+#endif
 #include <pthread.h>
 extern pthread_mutex_t _mm_mutex;
 #define MUTEX_LOCK   {pthread_mutex_lock(&_mm_mutex);
@@ -198,10 +209,12 @@ extern void      SL_Exit(SAMPLOAD*);
 
 extern void   UniSetRow(UBYTE*);
 extern UBYTE  UniGetByte(void);
+extern UWORD  UniGetWord(void);
 extern UBYTE* UniFindRow(UBYTE*,UWORD);
 extern void   UniSkipOpcode(UBYTE);
 extern void   UniReset(void);
-extern void   UniWrite(UBYTE);
+extern void   UniWriteByte(UBYTE);
+extern void   UniWriteWord(UWORD);
 extern void   UniNewline(void);
 extern UBYTE* UniDup(void);
 extern BOOL   UniInit(void);
@@ -220,60 +233,60 @@ enum {
 	/* Instrument change */
 	UNI_INSTRUMENT,
 	/* Protracker effects */
-	UNI_PTEFFECT0,
-	UNI_PTEFFECT1,
-	UNI_PTEFFECT2,
-	UNI_PTEFFECT3,
-	UNI_PTEFFECT4,
-	UNI_PTEFFECT5,
-	UNI_PTEFFECT6,
-	UNI_PTEFFECT7,
-	UNI_PTEFFECT8,
-	UNI_PTEFFECT9,
-	UNI_PTEFFECTA,
-	UNI_PTEFFECTB,
-	UNI_PTEFFECTC,
-	UNI_PTEFFECTD,
-	UNI_PTEFFECTE,
-	UNI_PTEFFECTF,
+	UNI_PTEFFECT0,     /* arpeggio */
+	UNI_PTEFFECT1,     /* porta up */
+	UNI_PTEFFECT2,     /* porta down */
+	UNI_PTEFFECT3,     /* porta to note */
+	UNI_PTEFFECT4,     /* vibrato */
+	UNI_PTEFFECT5,     /* dual effect 3+A */
+	UNI_PTEFFECT6,     /* dual effect 4+A */
+	UNI_PTEFFECT7,     /* tremolo */
+	UNI_PTEFFECT8,     /* pan */
+	UNI_PTEFFECT9,     /* sample offset */
+	UNI_PTEFFECTA,     /* volume slide */
+	UNI_PTEFFECTB,     /* pattern jump */
+	UNI_PTEFFECTC,     /* pattern break */
+	UNI_PTEFFECTD,     /* set volume */
+	UNI_PTEFFECTE,     /* extended effects */
+	UNI_PTEFFECTF,     /* set speed */
 	/* Scream Tracker effects */
-	UNI_S3MEFFECTA,
-	UNI_S3MEFFECTD,
-	UNI_S3MEFFECTE,
-	UNI_S3MEFFECTF,
-	UNI_S3MEFFECTI,
-	UNI_S3MEFFECTQ,
-	UNI_S3MEFFECTR,
-	UNI_S3MEFFECTT,
-	UNI_S3MEFFECTU, 
-	UNI_KEYOFF,
-	UNI_KEYFADE,
-	UNI_VOLEFFECTS,
+	UNI_S3MEFFECTA,    /* set speed */
+	UNI_S3MEFFECTD,    /* volume slide */
+	UNI_S3MEFFECTE,    /* porta down */
+	UNI_S3MEFFECTF,    /* porta up */
+	UNI_S3MEFFECTI,    /* tremor */
+	UNI_S3MEFFECTQ,    /* retrig */
+	UNI_S3MEFFECTR,    /* tremolo */
+	UNI_S3MEFFECTT,    /* set tempo */
+	UNI_S3MEFFECTU,    /* fine vibrato */
+	UNI_KEYOFF,        /* note off */
 	/* Fast Tracker effects */
-	UNI_XMEFFECT4,
-	UNI_XMEFFECTA,
-	UNI_XMEFFECTE1,
-	UNI_XMEFFECTE2,
-	UNI_XMEFFECTEA,
-	UNI_XMEFFECTEB,
-	UNI_XMEFFECTG,
-	UNI_XMEFFECTH,
-	UNI_XMEFFECTL,
-	UNI_XMEFFECTP,
-	UNI_XMEFFECTX1,
-	UNI_XMEFFECTX2,
+	UNI_KEYFADE,       /* note fade */
+	UNI_VOLEFFECTS,    /* volume column effects */
+	UNI_XMEFFECT4,     /* vibrato */
+	UNI_XMEFFECTA,     /* volume slide */
+	UNI_XMEFFECTE1,    /* fine porta up */
+	UNI_XMEFFECTE2,    /* fine porta down */
+	UNI_XMEFFECTEA,    /* fine volume slide up */
+	UNI_XMEFFECTEB,    /* fine volume slide down */
+	UNI_XMEFFECTG,     /* set global volume */
+	UNI_XMEFFECTH,     /* global volume slide */
+	UNI_XMEFFECTL,     /* set envelope position */
+	UNI_XMEFFECTP,     /* pan slide */
+	UNI_XMEFFECTX1,    /* extra fine porta up */
+	UNI_XMEFFECTX2,    /* extra fine porta down */
 	/* Impulse Tracker effects */
-	UNI_ITEFFECTG,     /* Porta to Note .. no kick=0; */
-	UNI_ITEFFECTH,     /* IT specific Vibrato */
-	UNI_ITEFFECTI,     /* IT tremor (xy not incremented) */
-	UNI_ITEFFECTM,     /* Set Channel Volume */
-	UNI_ITEFFECTN,     /* Slide / Fineslide Channel Volume */
-	UNI_ITEFFECTP,     /* Slide / Fineslide Channel Panning */
-	UNI_ITEFFECTT,     /* Slide tempo */
-	UNI_ITEFFECTU,     /* IT fine vibrato */
-	UNI_ITEFFECTW,     /* Slide / Fineslide Global volume */
-	UNI_ITEFFECTY,     /* The Satanic Panbrello */
-	UNI_ITEFFECTZ,     /* Resonant filters */
+	UNI_ITEFFECTG,     /* porta to note */
+	UNI_ITEFFECTH,     /* vibrato */
+	UNI_ITEFFECTI,     /* tremor (xy not incremented) */
+	UNI_ITEFFECTM,     /* set channel volume */
+	UNI_ITEFFECTN,     /* slide / fineslide channel volume */
+	UNI_ITEFFECTP,     /* slide / fineslide channel panning */
+	UNI_ITEFFECTT,     /* slide tempo */
+	UNI_ITEFFECTU,     /* fine vibrato */
+	UNI_ITEFFECTW,     /* slide / fineslide global volume */
+	UNI_ITEFFECTY,     /* panbrello */
+	UNI_ITEFFECTZ,     /* resonant filters */
 	UNI_ITEFFECTS0,
 	/* UltraTracker effects */
 	UNI_ULTEFFECT9,    /* Sample fine offset */
@@ -360,6 +373,11 @@ typedef struct FILTER {
 #define KEY_FADE        2
 #define KEY_KILL        (KEY_OFF|KEY_FADE)
 
+#define KICK_ABSENT     0
+#define KICK_NOTE       1
+#define KICK_KEYOFF     2
+#define KICK_ENV        4
+
 #define AV_IT           1   /* IT vs. XM vibrato info */
 
 /*========== Playing */
@@ -420,6 +438,8 @@ struct MP_VOICE*slave;        /* Audio Slave of current effects control channel
 	SWORD       tmpvolume;    /* tmp volume */
 	UWORD       tmpperiod;    /* tmp period */
 	UWORD       wantedperiod; /* period to slide to (with effect 3 or 5) */
+
+	UBYTE       arpmem;       /* arpeggio command memory */
 	UBYTE       pansspd;      /* panslide speed */
 	UWORD       slidespeed;   /* */
 	UWORD       portspeed;    /* noteslide speed (toneportamento) */
@@ -533,6 +553,7 @@ extern SBYTE   remap[64];           /* for removing empty channels */
 extern UBYTE*  poslookup;           /* lookup table for pattern jumps after
                                       blank pattern removal */
 extern UBYTE   poslookupcnt;
+extern UWORD*  origpositions;
 
 extern BOOL    filters;             /* resonant filters in use */
 extern UBYTE   activemacro;         /* active midi macro number for Sxx */
@@ -557,9 +578,10 @@ extern int*   AllocLinear(void);
 extern void   FreeLinear(void);
 extern int    speed_to_finetune(ULONG,int);
 extern void   S3MIT_ProcessCmd(UBYTE,UBYTE,BOOL);
+extern void   S3MIT_CreateOrders(BOOL);
 
 /* used to convert c4spd to linear XM periods (IT and IMF loaders). */
-extern UWORD  getlinearperiod(UBYTE,ULONG);
+extern UWORD  getlinearperiod(UWORD,ULONG);
 extern ULONG  getfrequency(UBYTE,ULONG);
 
 /*========== Player interface */
@@ -594,6 +616,14 @@ extern void   MD_SampleUnload(SWORD);
 extern ULONG  MD_SampleSpace(int);
 extern ULONG  MD_SampleLength(int,SAMPLE*);
 
+/* Parameter extraction helper */
+extern CHAR  *MD_GetAtom(CHAR*,CHAR*,BOOL);
+
+/* Internal software mixer stuff */
+extern void VC_SetupPointers(void);
+extern BOOL VC1_Init(void);
+extern BOOL VC2_Init(void);
+
 /*========== Prototypes for non-MT safe versions of some public functions */
 
 extern void _mm_registerdriver(struct MDRIVER*);
@@ -613,6 +643,10 @@ extern void Voice_SetVolume_internal(SBYTE,UWORD);
 extern void Voice_Stop_internal(SBYTE);
 extern BOOL Voice_Stopped_internal(SBYTE);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
 
 /* ex:set ts=4: */
