@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_669.c,v 1.24 1999/03/24 06:00:05 miod Exp $
+  $Id: load_669.c,v 1.28 1999/06/28 03:00:05 miod Exp $
 
   Composer 669 module loader
 
@@ -334,7 +334,7 @@ CHAR *S69_LoadTitle(void)
 MLOADER load_669={
 	NULL,
 	"669",
-	"669 loader v1.0",
+	"669 (Composer 669, Unis 669)",
 	S69_Init,
 	S69_Test,
 	S69_Load,
