@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_it.c,v 1.31 1999/03/24 06:00:05 miod Exp $
+  $Id: load_it.c,v 1.37 1999/06/28 03:00:05 miod Exp $
 
   Impulse tracker (IT) module loader
 
@@ -154,11 +154,13 @@ static ITNOTE *last=NULL;	/* uncompressing IT's pattern information */
 static int numtrk=0;
 static int old_effect;		/* if set, use S3M old-effects stuffs */
    
-static CHAR* IT_Version[4]={
+static CHAR* IT_Version[]={
 	"ImpulseTracker  .  ",
 	"Compressed ImpulseTracker  .  ",
 	"ImpulseTracker 2.14p3",
 	"Compressed ImpulseTracker 2.14p3",
+	"ImpulseTracker 2.14p4",
+	"Compressed ImpulseTracker 2.14p4",
 };
 
 /* table for porta-to-note command within volume/panning column */
@@ -196,6 +198,7 @@ void IT_Cleanup(void)
 	if(mask) free(mask);
 	if(last) free(last);
 	if(paraptr) free(paraptr);
+	if(origpositions) free(origpositions);
 
 	mh        = NULL;
 	poslookup = NULL;
@@ -203,6 +206,7 @@ void IT_Cleanup(void)
 	mask      = NULL;
 	last      = NULL;
 	paraptr   = NULL;
+	origpositions = NULL;
 }
 
 /* Because so many IT files have 64 channels as the set number used, but really
@@ -220,7 +224,7 @@ static BOOL IT_GetNumChannels(UWORD patrows)
 
 	do {
 		if((flag=_mm_read_UBYTE(modreader))==EOF) {
-			_mm_errno = MMERR_LOADING_PATTERN;
+			_mm_errno=MMERR_LOADING_PATTERN;
 			return 1;
 		}
 		if(!flag)
@@ -253,17 +257,18 @@ static UBYTE* IT_ConvertTrack(ITNOTE* tr,UWORD numrows)
 
 		if(note!=255) {
 			if(note==253)
-				UniWrite(UNI_KEYOFF);
-			else if(note==254)
+				UniWriteByte(UNI_KEYOFF);
+			else if(note==254) {
 				UniPTEffect(0xc,-1);	/* note cut command */
-			else
+				volpan=255;
+			} else
 				UniNote(note);
 		}
 
 		if((ins)&&(ins<100))
 			UniInstrument(ins-1);
 		else if(ins==253)
-			UniWrite(UNI_KEYOFF);
+			UniWriteByte(UNI_KEYOFF);
 		else if(ins!=255) { /* crap */
 			_mm_errno=MMERR_LOADING_PATTERN;
 			return NULL;
@@ -295,12 +300,12 @@ static UBYTE* IT_ConvertTrack(ITNOTE* tr,UWORD numrows)
 			UniVolEffect(VOL_PORTAMENTO,portatable[volpan-193]);
 		else if(volpan<=212)/* vibrato */
 			UniVolEffect(VOL_VIBRATO,(volpan-203));
-		else if(volpan!=255) { /* crap */
+		else if((volpan!=239)&&(volpan!=255)) { /* crap */
 			_mm_errno=MMERR_LOADING_PATTERN;
 			return NULL;
 		}
 
-		S3MIT_ProcessCmd(tr[t*of.numchn].cmd,tr[t*of.numchn].inf,old_effect);
+		S3MIT_ProcessCmd(tr[t*of.numchn].cmd,tr[t*of.numchn].inf,old_effect|2);
 
 		UniNewline();
 	}
@@ -461,7 +466,7 @@ BOOL IT_Load(BOOL curious)
 	_mm_read_UBYTES(mh->voltable,64,modreader);
 
 	if(_mm_eof(modreader)) {
-		_mm_errno = MMERR_LOADING_HEADER;
+		_mm_errno=MMERR_LOADING_HEADER;
 		return 0;
 	}
 
@@ -474,13 +479,22 @@ BOOL IT_Load(BOOL curious)
 	of.initspeed   = mh->initspeed;
 	of.inittempo   = mh->inittempo;
 	of.initvolume  = mh->globvol;
-	of.flags      |= UF_BGSLIDES;
+	of.flags      |= UF_BGSLIDES|UF_ARPMEM;
+
+	if(mh->songname[25]) {
+		of.numvoices=1+mh->songname[25];
+#ifdef MIKMOD_DEBUG
+		fprintf(stderr,"Embedded IT limitation to %d voices\n",of.numvoices);
+#endif
+	}
 
 	/* set the module type */
-	/* 2.17 : don't know but found one module... */
+	/* 2.17 : IT 2.14p4 */
 	/* 2.16 : IT 2.14p3 with resonant filters */
 	/* 2.15 : IT 2.14p3 (improved compression) */
-	if((mh->cwt<=0x219)&&(mh->cwt>=0x215))
+	if((mh->cwt<=0x219)&&(mh->cwt>=0x217))
+		of.modtype=strdup(IT_Version[mh->cmwt<0x214?4:5]);
+	else if (mh->cwt>=0x215)
 		of.modtype=strdup(IT_Version[mh->cmwt<0x214?2:3]);
 	else {
 		of.modtype     = strdup(IT_Version[mh->cmwt<0x214?0:1]);
@@ -489,13 +503,13 @@ BOOL IT_Load(BOOL curious)
 		of.modtype[mh->cmwt<0x214?18:29] = ((mh->cwt)&0xf)+'0';
 	}
 
-	old_effect = 0;
-	if(mh->flags&8) {
+	if(mh->flags&8)
 		of.flags|=(UF_XMPERIODS | UF_LINEAR);
-		old_effect|=2;
-	}
+
 	if((mh->cwt>=0x106)&&(mh->flags&16))
-		old_effect|=1;
+		old_effect=1;
+	else
+		old_effect=0;
 
 	/* set panning positions */
 	for(t=0;t<64;t++) {
@@ -519,11 +533,12 @@ BOOL IT_Load(BOOL curious)
 
 	/* read the order data */
 	if(!AllocPositions(mh->ordnum)) return 0;
+	if(!(origpositions=_mm_calloc(mh->ordnum,sizeof(UWORD)))) return 0;
 
 	for(t=0;t<mh->ordnum;t++) {
-		of.positions[t]=_mm_read_UBYTE(modreader);
-		if((of.positions[t]>mh->patnum)&&(of.positions[t]<254))
-			of.positions[t]=255;
+		origpositions[t]=_mm_read_UBYTE(modreader);
+		if((origpositions[t]>mh->patnum)&&(origpositions[t]<254))
+			origpositions[t]=255;
 	}
 
 	if(_mm_eof(modreader)) {
@@ -531,15 +546,8 @@ BOOL IT_Load(BOOL curious)
 		return 0;
 	}
 
-	of.numpos = 0;poslookupcnt=mh->ordnum;
-	for(t=0;t<mh->ordnum;t++) {
-		of.positions[of.numpos]=of.positions[t];
-		poslookup[t]=of.numpos;	/* bug fix for freaky S3Ms / ITs */
-		if(of.positions[t]<254) of.numpos++;
-		else
-		  /* end of song special pattern */
-		  if((of.positions[t]==255)&&(!curious)) break;
-	}
+	poslookupcnt=mh->ordnum;
+	S3MIT_CreateOrders(curious);
 
 	if(!(paraptr=(ULONG*)_mm_malloc((mh->insnum+mh->smpnum+of.numpat)*
 	                               sizeof(ULONG)))) return 0;
@@ -547,7 +555,6 @@ BOOL IT_Load(BOOL curious)
 	/* read the instrument, sample, and pattern parapointers */
 	_mm_read_I_ULONGS(paraptr,mh->insnum+mh->smpnum+of.numpat,modreader);
 
-	/* now is a good time to check if the header was too short */
 	if(_mm_eof(modreader)) {
 		_mm_errno = MMERR_LOADING_HEADER;
 		return 0;
@@ -815,6 +822,7 @@ BOOL IT_Load(BOOL curious)
 				if(ih.pitflg&0x80) {
 					/* filter envelopes not supported yet */
 					d->pitflg&=~EF_ON;
+					ih.pitpts=ih.pitbeg=ih.pitend=0;
 #ifdef MIKMOD_DEBUG
 					fputs("\rFilter envelopes not supported yet\n",stderr);
 #endif
@@ -895,8 +903,8 @@ BOOL IT_Load(BOOL curious)
 			_mm_fseek(modreader,((long)paraptr[mh->insnum+mh->smpnum+t]),SEEK_SET);
 			_mm_read_I_UWORD(modreader);
 			/* read pattern length (# of rows) */
-			packlen = _mm_read_I_UWORD(modreader);
-			if((packlen<32)||(packlen>200)) {
+			packlen=_mm_read_I_UWORD(modreader);
+			if((packlen<32)||(packlen>256)) {
 				_mm_errno=MMERR_LOADING_PATTERN;
 				return 0;
 			}
@@ -911,6 +919,8 @@ BOOL IT_Load(BOOL curious)
 			remap[t]=of.numchn++;
 
 	of.numtrk = of.numpat*of.numchn;
+	if(of.numvoices)
+		if (of.numvoices<of.numchn) of.numvoices=of.numchn;
 
 	if(!AllocPatterns()) return 0;
 	if(!AllocTracks()) return 0;
@@ -955,7 +965,7 @@ CHAR *IT_LoadTitle(void)
 MLOADER load_it={
 	NULL,
 	"IT",
-	"IT loader v1.0",
+	"IT (Impulse Tracker)",
 	IT_Init,
 	IT_Test,
 	IT_Load,
