@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_s3m.c,v 1.24 1999/03/24 06:00:05 miod Exp $
+  $Id: load_s3m.c,v 1.30 1999/06/28 03:00:05 miod Exp $
 
   Screamtracker (S3M) module loader
 
@@ -90,16 +90,17 @@ static S3MHEADER *mh      = NULL;
 static UWORD     *paraptr = NULL; /* parapointer array (see S3M docs) */
 
 /* tracker identifiers */
-#define NUMTRACKERS (5)
-static CHAR* S3M_Version[NUMTRACKERS] = {
+#define NUMTRACKERS 4
+static CHAR* S3M_Version[] = {
 	"Screamtracker x.xx",
 	"Imago Orpheus x.xx (S3M format)",
 	"Impulse Tracker x.xx (S3M format)",
 	"Unknown tracker x.xx (S3M format)",
 	"Impulse Tracker 2.14p3 (S3M format)"
+	"Impulse Tracker 2.14p4 (S3M format)"
 };
 /* version number position in above array */
-static int numeric[NUMTRACKERS-1]={14,14,16,16};
+static int numeric[NUMTRACKERS]={14,14,16,16};
 
 /*========== Loader code */
 
@@ -118,6 +119,7 @@ BOOL S3M_Init(void)
 	if(!(s3mbuf=(S3MNOTE*)_mm_malloc(32*64*sizeof(S3MNOTE)))) return 0;
 	if(!(mh=(S3MHEADER*)_mm_malloc(sizeof(S3MHEADER)))) return 0;
 	if(!(poslookup=(UBYTE*)_mm_malloc(sizeof(UBYTE)*256))) return 0;
+	memset(poslookup,-1,256);
 
 	return 1;
 }
@@ -128,11 +130,13 @@ void S3M_Cleanup(void)
 	if(paraptr) free(paraptr);
 	if(poslookup) free(poslookup);
 	if(mh) free(mh);
+	if(origpositions) free(origpositions);
 
 	paraptr   = NULL;
 	s3mbuf    = NULL;
 	poslookup = NULL;
 	mh        = NULL;
+	origpositions = NULL;
 }
 
 /* Because so many s3m files have 16 channels as the set number used, but really
@@ -222,8 +226,11 @@ static UBYTE* S3M_ConvertTrack(S3MNOTE* tr)
 
 		if((ins)&&(ins!=255)) UniInstrument(ins-1);
 		if(note!=255) {
-			if(note==254) UniPTEffect(0xc,0);	/* note cut command */
-			else UniNote(((note>>4)*OCTAVE)+(note&0xf));	/* normal note */
+			if(note==254) {
+				UniPTEffect(0xc,0);	/* note cut command */
+				vol=255;
+			} else
+				UniNote(((note>>4)*OCTAVE)+(note&0xf));	/* normal note */
 		}
 		if(vol<255) UniPTEffect(0xc,vol);
 
@@ -274,24 +281,28 @@ BOOL S3M_Load(BOOL curious)
 	of.initspeed   = mh->initspeed;
 	of.inittempo   = mh->inittempo;
 	of.initvolume  = mh->mastervol<<1;
+	of.flags      |= UF_ARPMEM;
 	if((mh->tracker==0x1300)||(mh->flags&64))
 		of.flags|=UF_S3MSLIDES;
 
 	/* read the order data */
 	if(!AllocPositions(mh->ordnum)) return 0;
-	for(t=0;t<mh->ordnum;t++)
-		of.positions[t]=_mm_read_UBYTE(modreader);
+	if(!(origpositions=_mm_calloc(mh->ordnum,sizeof(UWORD)))) return 0;
 
-	of.numpos=0;poslookupcnt=mh->ordnum;
 	for(t=0;t<mh->ordnum;t++) {
-		of.positions[of.numpos]=of.positions[t];
-		poslookup[t]=of.numpos;	/* bug fix for freaky S3Ms */
-		if(of.positions[t]<254) of.numpos++;        
-		else
-		  /* special end of song pattern */
-		  if((of.positions[t]==255)&&(!curious)) break;
+		origpositions[t]=_mm_read_UBYTE(modreader);
+		if((origpositions[t]>=mh->patnum)&&(origpositions[t]<254))
+			origpositions[t]=255/*mh->patnum-1*/;
 	}
 
+	if(_mm_eof(modreader)) {
+		_mm_errno = MMERR_LOADING_HEADER;
+		return 0;
+	}
+
+	poslookupcnt=mh->ordnum;
+	S3MIT_CreateOrders(curious);
+
 	if(!(paraptr=(UWORD*)_mm_malloc((of.numins+of.numpat)*sizeof(UWORD))))
 		return 0;
 
@@ -373,14 +384,16 @@ BOOL S3M_Load(BOOL curious)
 
 	/* then we can decide the module type */
 	t=mh->tracker>>12;
-	if((!t)||(t>3)) t=NUMTRACKERS-1; /* unknown tracker */
-	if(mh->tracker>=0x3216)
-		t=NUMTRACKERS; /* IT 2.14p3 */
-	else
-	  if(mh->tracker==0x3215)
-		mh->tracker=0x3214; /* lie about IT 2.14p1 or p2, report 2.14 */
+	if((!t)||(t>3))
+		t=NUMTRACKERS-1; /* unknown tracker */
+	else {
+		if(mh->tracker>=0x3217)
+			t=NUMTRACKERS+1; /* IT 2.14p4 */
+		else if(mh->tracker>=0x3217)
+			t=NUMTRACKERS; /* IT 2.14p3 */
+	}
 	of.modtype = strdup(S3M_Version[t-1]);
-	if(t!=NUMTRACKERS) {
+	if(t<NUMTRACKERS) {
 		of.modtype[numeric[t-1]] = ((mh->tracker>>8) &0xf)+'0';
 		of.modtype[numeric[t-1]+2] = ((mh->tracker>>4)&0xf)+'0';
 		of.modtype[numeric[t-1]+3] = ((mh->tracker)&0xf)+'0';
@@ -436,7 +449,7 @@ CHAR *S3M_LoadTitle(void)
 MLOADER load_s3m={
 	NULL,
 	"S3M",
-	"S3M loader v1.0",
+	"S3M (Scream Tracker 3)",
 	S3M_Init,
 	S3M_Test,
 	S3M_Load,
