@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_uni.c,v 1.6 1999/03/24 06:00:05 miod Exp $
+  $Id: load_uni.c,v 1.11 1999/06/28 03:00:05 miod Exp $
 
   UNIMOD (libmikmod's and APlayer's internal module format) loader
 
@@ -185,21 +185,27 @@ static BOOL loadsmp6(void)
 
 		flags         = _mm_read_M_UWORD(modreader);
 		/* convert flags */
-		if(universion>=0x102)
-			s->flags=flags;
-		else {
+		if(universion>=0x102) {
+			s->flags=flags&SF_FORMATMASK;
+			if(flags&0x0800) s->flags|=SF_UST_LOOP;
+			if(flags&0x0400) s->flags|=SF_OWNPAN;
+			if(flags&0x0200) s->flags|=SF_SUSTAIN;
+			if(flags&0x0100) s->flags|=SF_REVERSE;
+			if(flags&0x0080) s->flags|=SF_BIDI;
+			if(flags&0x0040) s->flags|=SF_LOOP;
+		} else {
 			s->flags=0;
-			if(flags&1024) s->flags|=SF_UST_LOOP;
-			if(flags& 512) s->flags|=SF_OWNPAN;
-			if(flags& 256) s->flags|=SF_REVERSE;
-			if(flags& 128) s->flags|=SF_SUSTAIN;
-			if(flags&  64) s->flags|=SF_BIDI;
-			if(flags&  32) s->flags|=SF_LOOP;
-			if(flags&  16) s->flags|=SF_BIG_ENDIAN;
-			if(flags&   8) s->flags|=SF_DELTA;
-			if(flags&   4) s->flags|=SF_STEREO;
-			if(flags&   2) s->flags|=SF_SIGNED;
-			if(flags&   1) s->flags|=SF_16BITS;
+			if(flags&0x400) s->flags|=SF_UST_LOOP;
+			if(flags&0x200) s->flags|=SF_OWNPAN;
+			if(flags&0x100) s->flags|=SF_REVERSE;
+			if(flags&0x080) s->flags|=SF_SUSTAIN;
+			if(flags&0x040) s->flags|=SF_BIDI;
+			if(flags&0x020) s->flags|=SF_LOOP;
+			if(flags&0x010) s->flags|=SF_BIG_ENDIAN;
+			if(flags&0x008) s->flags|=SF_DELTA;
+			if(flags&0x004) s->flags|=SF_STEREO;
+			if(flags&0x002) s->flags|=SF_SIGNED;
+			if(flags&0x001) s->flags|=SF_16BITS;
 		}
 
 		s->speed      = _mm_read_M_ULONG(modreader);
@@ -498,11 +504,20 @@ BOOL UNI_Load(BOOL curious)
 	if(universion<0x102)
 		oldtype=readstring();
 	if(oldtype) {
-		if(!(modtype=_mm_malloc(strlen(oldtype)+20))) return 0;
-		sprintf(modtype,"%s (was %s)",(universion>=0x100)?"APlayer":"MikCvt",oldtype);
+		int len=strlen(oldtype)+20;
+		if(!(modtype=_mm_malloc(len))) return 0;
+#ifdef HAVE_SNPRINTF
+		snprintf(modtype,len,"%s (was %s)",(universion>=0x100)?"APlayer":"MikCvt2",oldtype);
+#else
+		sprintf(modtype,"%s (was %s)",(universion>=0x100)?"APlayer":"MikCvt2",oldtype);
+#endif
 	} else {
 		if(!(modtype=_mm_malloc(10))) return 0;
-		sprintf(modtype,"%s",(universion>=0x100)?"APlayer":"MikCvt");
+#ifdef HAVE_SNPRINTF
+		snprintf(modtype,10,"%s",(universion>=0x100)?"APlayer":"MikCvt3");
+#else
+		sprintf(modtype,"%s",(universion>=0x100)?"APlayer":"MikCvt3");
+#endif
 	}
 	of.modtype=strdup(modtype);
 	free(modtype);free(oldtype);
@@ -615,7 +630,7 @@ CHAR *UNI_LoadTitle(void)
 MLOADER load_uni={
 	NULL,
 	"UNI",
-	"APUN and UNI loader v1.0",
+	"APUN (APlayer) and UNI (MikMod)",
 	UNI_Init,
 	UNI_Test,
 	UNI_Load,
