@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_xm.c,v 1.24 1999/03/24 06:00:05 miod Exp $
+  $Id: load_xm.c,v 1.30 1999/06/28 03:00:05 miod Exp $
 
   Fasttracker (XM) module loader
 
@@ -184,7 +184,7 @@ static UBYTE* XM_Convert(XMNOTE* xmtrack,UWORD rows)
 		dat  = xmtrack->dat;
 
 		if(note) {
-			if(note>=8*OCTAVE+1)
+			if(note>XMNOTECNT)
 				UniEffect(UNI_KEYFADE,0);
 			else
 				UniNote(note-1);
@@ -232,7 +232,7 @@ static UBYTE* XM_Convert(XMNOTE* xmtrack,UWORD rows)
 		}
 
 		switch(eff) {
-			case 0x4: /* Effect 4: Vibrato */
+			case 0x4:
 				UniEffect(UNI_XMEFFECT4,dat);
 				break;
 			case 0xa:
@@ -253,7 +253,7 @@ static UBYTE* XM_Convert(XMNOTE* xmtrack,UWORD rows)
 						UniEffect(UNI_XMEFFECTEB,dat&0xf);
 						break;
 					default:
-						UniPTEffect(0x0e,dat);
+						UniPTEffect(eff,dat);
 				}
 				break;
 			case 'G'-55: /* G - set global volume */
@@ -288,10 +288,15 @@ static UBYTE* XM_Convert(XMNOTE* xmtrack,UWORD rows)
 				}
 				break;
 			default:
-				if(!(eff>>4)) {
-					/* Convert pattern jump from Dec to Hex */
-					if(eff==0xd)
-						dat=(((dat&0xf0)>>4)*10)+(dat&0xf);
+				if(eff<=0xf) {
+					/* the pattern jump destination is written in decimal,
+					   but it seems some poor tracker software writes them
+					   in hexadecimal... (sigh) */
+					if (eff==0xd)
+						/* don't change anything if we're sure it's in hexa */
+						if ((((dat&0xf0)>>4)<=9)&&((dat&0xf)<=9))
+							/* otherwise, convert from dec to hex */
+							dat=(((dat&0xf0)>>4)*10)+(dat&0xf);
 					UniPTEffect(eff,dat);
 				}
 				break;
@@ -484,7 +489,7 @@ static BOOL LoadInstruments(void)
 				   have to load them all into a temp area, count the of.numsmp
 				   along the way and then do an AllocSamples() and move
 				   everything over */
-				if(mh->version>0x0102) next = 0;
+				if(mh->version>0x0103) next = 0;
 				for(u=0;u<ih.numsmp;u++,s++) {
 					/* Allocate more room for sample information if necessary */
 					if(of.numsmp+u==wavcnt) {
@@ -528,7 +533,7 @@ static BOOL LoadInstruments(void)
 					}
 				}
 
-				if(mh->version>0x0102) {
+				if(mh->version>0x0103) {
 					for(u=0;u<ih.numsmp;u++)
 						nextwav[of.numsmp++]+=_mm_ftell(modreader);
 					_mm_fseek(modreader,next,SEEK_CUR);
@@ -600,9 +605,19 @@ BOOL XM_Load(BOOL curious)
 	of.initspeed = mh->tempo;         
 	of.inittempo = mh->bpm;
 	strncpy(tracker,mh->trackername,20);tracker[20]=0;
-	for(t=20;tracker[t]<=' ';t--) tracker[t]=0;
+	for(t=20;(tracker[t]<=' ')&&(t>=0);t--) tracker[t]=0;
+	
+	/* some modules have the tracker name empty */
+	if (!tracker[0])
+		strcpy(tracker,"Unknown tracker");
+
+#ifdef HAVE_SNPRINTF
+	snprintf(modtype,60,"%s (XM format %d.%02d)",
+	                    tracker,mh->version>>8,mh->version&0xff);
+#else
 	sprintf(modtype,"%s (XM format %d.%02d)",
 	                tracker,mh->version>>8,mh->version&0xff);
+#endif
 	of.modtype   = strdup(modtype);
 	of.numchn    = mh->numchn;
 	of.numpat    = mh->numpat;
@@ -633,7 +648,7 @@ BOOL XM_Load(BOOL curious)
 		of.numpat++;of.numtrk+=of.numchn;
 	}
 
-	if(mh->version==0x0102) {
+	if(mh->version<0x0104) {
 		if(!LoadInstruments()) return 0;
 		if(!LoadPatterns(dummypat)) return 0;
 		for(t=0;t<of.numsmp;t++)
@@ -679,11 +694,15 @@ BOOL XM_Load(BOOL curious)
 
 	d=of.instruments;
 	s=wh;
-	for(u=0;u<of.numins;u++,d++) {
-		for(t=0;t<XMNOTECNT;t++)
-			d->samplenote[t]=(d->samplenumber[t]>=of.numsmp)?
-			  255:(t+s[d->samplenumber[t]].relnote);
-	}
+	for(u=0;u<of.numins;u++,d++)
+		for(t=0;t<XMNOTECNT;t++) {
+			if (d->samplenumber[t]>=of.numsmp)
+				d->samplenote[t]=255;
+			else {
+				int note=t+s[d->samplenumber[t]].relnote;
+				d->samplenote[t]=(note<0)?0:note;
+			}
+		}
 
 	free(wh);free(nextwav);
 	wh=NULL;nextwav=NULL;
@@ -705,7 +724,7 @@ CHAR *XM_LoadTitle(void)
 MLOADER load_xm={
 	NULL,
 	"XM",
-	"XM loader v1.0",
+	"XM (FastTracker 2)",
 	XM_Init,
 	XM_Test,
 	XM_Load,
