@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mdreg.c,v 1.23 1999/03/24 06:00:06 miod Exp $
+  $Id: mdreg.c,v 1.28 1999/06/28 03:00:05 miod Exp $
 
   Routine for registering all drivers in libmikmod for the current platform.
 
@@ -70,19 +70,26 @@ void _mm_registeralldrivers(void)
 	_mm_registerdriver(&drv_dart);
 #endif
 #ifdef DRV_OS2
-	_mm_registerdriver(&drv_os2s);
-	_mm_registerdriver(&drv_os2l);
+	_mm_registerdriver(&drv_os2);
 #endif
 #ifdef DRV_DS
 	_mm_registerdriver(&drv_ds);
 #endif
+#ifdef DRV_MAC
+	_mm_registerdriver(&drv_mac);
+#endif
 
 	/* Register disk writers */
 	_mm_registerdriver(&drv_raw);
 	_mm_registerdriver(&drv_wav);
 
 	/* Register other drivers */
+#ifdef DRV_PIPE
+	_mm_registerdriver(&drv_pipe);
+#endif
+#ifndef macintosh
 	_mm_registerdriver(&drv_stdout);
+#endif
 
 	_mm_registerdriver(&drv_nos);
 }
