@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mloader.c,v 1.26 1999/03/24 06:00:06 miod Exp $
+  $Id: mloader.c,v 1.31 1999/06/28 03:00:05 miod Exp $
 
   These routines are used to access the available module loaders
 
@@ -46,20 +46,20 @@ UWORD finetune[16]={
 
 CHAR* MikMod_InfoLoader(void)
 {
-	int t,len=0;
+	int len=0;
 	MLOADER *l;
 	CHAR *list=NULL;
 
 	MUTEX_LOCK
 		/* compute size of buffer */
-		for(l=firstloader;l;l=l->next) len+=4+(l->next?1:0)+strlen(l->version);
+		for(l=firstloader;l;l=l->next) len+=1+(l->next?1:0)+strlen(l->version);
 
 		if(len)
 			if((list=_mm_malloc(len*sizeof(CHAR)))) {
 				list[0]=0;
 				/* list all registered module loders */
-				for(t=1,l=firstloader;l;l=l->next,t++)
-					sprintf(list,(l->next)?"%s%2d %s\n":"%s%2d %s",list,t,l->version);
+				for(l=firstloader;l;l=l->next)
+					sprintf(list,(l->next)?"%s%s\n":"%s%s",list,l->version);
 			}
 	MUTEX_UNLOCK
 	return list;
@@ -257,7 +257,7 @@ CHAR *DupStr(CHAR* s,UWORD len,BOOL strict)
 
 	/* When the buffer wasn't completely empty, allocate a cstring and copy the
 	   buffer into that string, except for any control-chars */
-	if((d=(CHAR*)_mm_malloc(len+1))) {
+	if((d=(CHAR*)_mm_malloc(sizeof(CHAR)*(len+1)))) {
 		for(t=0;t<len;t++) d[t]=(s[t]<32)?'.':s[t];
 		d[len]=0;
 	}
@@ -498,7 +498,7 @@ MODULE* Player_LoadFP(FILE* fp,int maxchan,BOOL curious)
 	return result;
 }
 
-/* Open a module via it's filename.  The loader will initialize the specified
+/* Open a module via its filename.  The loader will initialize the specified
    song-player 'player'. */
 MODULE* Player_Load(CHAR* filename,int maxchan,BOOL curious)
 {
@@ -506,7 +506,7 @@ MODULE* Player_Load(CHAR* filename,int maxchan,BOOL curious)
 	MODULE *mf=NULL;
 
 	if((fp=_mm_fopen(filename,"rb"))) {
-		mf=Player_LoadFP(fp, maxchan,curious);
+		mf=Player_LoadFP(fp,maxchan,curious);
 		fclose(fp);
 	}
 	return mf;
