@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: virtch.c,v 1.25 1999/03/24 06:00:06 miod Exp $
+  $Id: virtch.c,v 1.31 1999/06/28 03:00:06 miod Exp $
 
   Sample mixing routines, using a 32 bits mixing buffer.
 
@@ -43,8 +43,6 @@
 #include <stddef.h>
 #include <string.h>
 
-#define __FAST_REVERB__
-
 /*
    Constant definitions
    ====================
@@ -109,23 +107,16 @@ static	UWORD vc_mode;
 
 /* Reverb control variables */
 
-static	int RVc1, RVc2, RVc3, RVc4;
-#ifndef __FAST_REVERB__
-static	int RVc5, RVc6, RVc7, RVc8;
-#endif
+static	int RVc1, RVc2, RVc3, RVc4, RVc5, RVc6, RVc7, RVc8;
 static	ULONG RVRindex;
 
 /* For Mono or Left Channel */
-static	SLONG *RVbufL1=NULL,*RVbufL2=NULL,*RVbufL3=NULL,*RVbufL4=NULL;
-#ifndef __FAST_REVERB__
-static	SLONG *RVbufL5=NULL,*RVbufL6=NULL,*RVbufL7=NULL,*RVbufL8=NULL;
-#endif
+static	SLONG *RVbufL1=NULL,*RVbufL2=NULL,*RVbufL3=NULL,*RVbufL4=NULL,
+		      *RVbufL5=NULL,*RVbufL6=NULL,*RVbufL7=NULL,*RVbufL8=NULL;
 
 /* For Stereo only (Right Channel) */
-static	SLONG *RVbufR1=NULL,*RVbufR2=NULL,*RVbufR3=NULL,*RVbufR4=NULL;
-#ifndef __FAST_REVERB__
-static	SLONG *RVbufR5=NULL,*RVbufR6=NULL,*RVbufR7=NULL,*RVbufR8=NULL;
-#endif
+static	SLONG *RVbufR1=NULL,*RVbufR2=NULL,*RVbufR3=NULL,*RVbufR4=NULL,
+		      *RVbufR5=NULL,*RVbufR6=NULL,*RVbufR7=NULL,*RVbufR8=NULL;
 
 #ifdef NATIVE_64BIT_INT
 #define NATIVE SLONGLONG
@@ -187,15 +178,15 @@ static SLONG Mix32MonoInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increment
 	SLONG sample;
 
 	while(todo--) {
-		sample=((srce[index>>FRACBITS]*((FRACMASK+1L)-(index&FRACMASK))) +
-		        (srce[(index>>FRACBITS)+1]*(index&FRACMASK))) >> FRACBITS;
+		sample=(SLONG)srce[index>>FRACBITS]+
+		       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
 		if(vnf->rampvol) {
-			*dest++ += (
-			  ((SLONG)(vnf->oldlvol*vnf->rampvol)+
-			   (vnf->lvolsel*(CLICK_BUFFER-vnf->rampvol))
-			  )*sample)>>CLICK_SHIFT;
+			*dest++ += (((SLONG)vnf->oldlvol<<CLICK_SHIFT) +
+			            (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
+			           )*sample>>CLICK_SHIFT;
 			vnf->rampvol--;
 		} else
 			*dest++ += vnf->lvolsel * sample;
@@ -208,19 +199,18 @@ static SLONG Mix32StereoInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG increme
 	SLONG sample;
 
 	while(todo--) {
-		sample=((srce[index>>FRACBITS]*((FRACMASK+1L)-(index&FRACMASK))) +
-		        (srce[(index>>FRACBITS)+1]*(index&FRACMASK))) >> FRACBITS;
+		sample=(SLONG)srce[index>>FRACBITS]+
+		       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
 		if(vnf->rampvol) {
-			*dest++ += (
-			  ((SLONG)(vnf->oldlvol*vnf->rampvol)+
-			   (vnf->lvolsel*(CLICK_BUFFER-vnf->rampvol))
-			  )*sample)>>CLICK_SHIFT;
-			*dest++ += (
-			  ((SLONG)(vnf->oldrvol*vnf->rampvol)+
-			   (vnf->rvolsel*(CLICK_BUFFER-vnf->rampvol))
-			  )*sample)>>CLICK_SHIFT;
+			*dest++ += (((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
+			            (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
+			           )*sample>>CLICK_SHIFT;
+			*dest++ += (((SLONG)vnf->oldrvol<<CLICK_SHIFT)+
+			            (SLONG)(vnf->oldrvol-vnf->rvolsel)*vnf->rampvol
+			           )*sample>>CLICK_SHIFT;
 			vnf->rampvol--;
 		} else {
 			*dest++ += vnf->lvolsel * sample;
@@ -237,8 +227,9 @@ static SLONG Mix32SurroundInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG incre
 	while(todo--) {
 		int oldvol,vol;
 
-		sample=((srce[index>>FRACBITS]*((FRACMASK+1L)-(index&FRACMASK))) +
-		        (srce[(index>>FRACBITS)+1]*(index&FRACMASK))) >> FRACBITS;
+		sample=(SLONG)srce[index>>FRACBITS]+
+		       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
 		if(vnf->lvolsel>=vnf->rvolsel) {
@@ -248,9 +239,9 @@ static SLONG Mix32SurroundInterp(SWORD* srce,SLONG* dest,SLONG index,SLONG incre
 		}
 
 		if(vnf->rampvol) {
-			sample = (
-			  ((SLONG)(oldvol*vnf->rampvol)+(vol*(CLICK_BUFFER-vnf->rampvol))
-			  )*sample)>>CLICK_SHIFT;
+			sample=(((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
+			        (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
+			       )*sample>>CLICK_SHIFT;
 			*dest++ += sample;
 			*dest++ -= sample;
 			vnf->rampvol--;
@@ -316,15 +307,15 @@ static SLONGLONG MixMonoInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLONG
 	SLONG sample;
 
 	while(todo--) {
-		sample=((srce[index>>FRACBITS]*((FRACMASK+1L)-(index&FRACMASK))) +
-		        (srce[(index>>FRACBITS)+1]*(index&FRACMASK))) >> FRACBITS;
+		sample=(SLONG)srce[index>>FRACBITS]+
+		       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
 		if(vnf->rampvol) {
-			*dest++ += (
-			  ((SLONG)(vnf->oldlvol*vnf->rampvol)+
-			   (vnf->lvolsel*(CLICK_BUFFER-vnf->rampvol))
-			  )*sample)>>CLICK_SHIFT;
+			*dest++ += (((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
+			            (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
+			           )*sample>>CLICK_SHIFT;
 			vnf->rampvol--;
 		} else
 			*dest++ += vnf->lvolsel * sample;
@@ -337,19 +328,18 @@ static SLONGLONG MixStereoInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONGLO
 	SLONG sample;
 
 	while(todo--) {
-		sample=((srce[index>>FRACBITS]*((FRACMASK+1L)-(index&FRACMASK))) +
-		        (srce[(index>>FRACBITS)+1]*(index&FRACMASK))) >> FRACBITS;
+		sample=(SLONG)srce[index>>FRACBITS]+
+		       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
 		if(vnf->rampvol) {
-			*dest++ += (
-			  ((SLONG)(vnf->oldlvol*vnf->rampvol)+
-			   (vnf->lvolsel*(CLICK_BUFFER-vnf->rampvol))
-			  )*sample)>>CLICK_SHIFT;
-			*dest++ += (
-			  ((SLONG)(vnf->oldrvol*vnf->rampvol)+
-			   (vnf->rvolsel*(CLICK_BUFFER-vnf->rampvol))
-			  )*sample)>>CLICK_SHIFT;
+			*dest++ +=(((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
+			           (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
+			          )*sample>>CLICK_SHIFT;
+			*dest++ +=(((SLONG)vnf->oldrvol<<CLICK_SHIFT)+
+			           (SLONG)(vnf->oldrvol-vnf->rvolsel)*vnf->rampvol
+			          )*sample>>CLICK_SHIFT;
 			vnf->rampvol--;
 		} else {
 			*dest++ += vnf->lvolsel * sample;
@@ -366,8 +356,9 @@ static SLONGLONG MixSurroundInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONG
 	while(todo--) {
 		int oldvol,vol;
 
-		sample=((srce[index>>FRACBITS]*((FRACMASK+1L)-(index&FRACMASK))) +
-		        (srce[(index>>FRACBITS)+1]*(index&FRACMASK))) >> FRACBITS;
+		sample=(SLONG)srce[index>>FRACBITS]+
+		       ((SLONG)(srce[(index>>FRACBITS)+1]-srce[index>>FRACBITS])
+		        *(index&FRACMASK)>>FRACBITS);
 		index += increment;
 
 		if(vnf->lvolsel>=vnf->rvolsel) {
@@ -377,9 +368,9 @@ static SLONGLONG MixSurroundInterp(SWORD* srce,SLONG* dest,SLONGLONG index,SLONG
 		}
 
 		if(vnf->rampvol) {
-			sample = (
-			  ((SLONG)(oldvol*vnf->rampvol)+(vol*(CLICK_BUFFER-vnf->rampvol))
-			  )*sample)>>CLICK_SHIFT;
+			sample=(((SLONG)vnf->oldlvol<<CLICK_SHIFT)+
+			        (SLONG)(vnf->oldlvol-vnf->lvolsel)*vnf->rampvol
+			       )*sample>>CLICK_SHIFT;
 			*dest++ += sample;
 			*dest++ -= sample;
 			vnf->rampvol--;
@@ -403,43 +394,29 @@ static void MixReverb_Normal(SLONG* srce,NATIVE count)
 	unsigned int speedup;
 	int ReverbPct;
 	unsigned int loc1,loc2,loc3,loc4;
-#ifndef __FAST_REVERB__
 	unsigned int loc5,loc6,loc7,loc8;
 
-	ReverbPct=58+(md_reverb*4);
-#else
-	ReverbPct=89+(md_reverb*2);
-#endif
+	ReverbPct=58+(md_reverb<<2);
 
 	COMPUTE_LOC(1); COMPUTE_LOC(2); COMPUTE_LOC(3); COMPUTE_LOC(4);
-#ifndef __FAST_REVERB__
 	COMPUTE_LOC(5); COMPUTE_LOC(6); COMPUTE_LOC(7); COMPUTE_LOC(8);
-#endif
 
 	while(count--) {
 		/* Compute the left channel echo buffers */
 		speedup = *srce >> 3;
 
 		COMPUTE_LECHO(1); COMPUTE_LECHO(2); COMPUTE_LECHO(3); COMPUTE_LECHO(4);
-#ifndef __FAST_REVERB__
 		COMPUTE_LECHO(5); COMPUTE_LECHO(6); COMPUTE_LECHO(7); COMPUTE_LECHO(8);
-#endif
 
 		/* Prepare to compute actual finalized data */
 		RVRindex++;
 
 		COMPUTE_LOC(1); COMPUTE_LOC(2); COMPUTE_LOC(3); COMPUTE_LOC(4);
-#ifndef __FAST_REVERB__
 		COMPUTE_LOC(5); COMPUTE_LOC(6); COMPUTE_LOC(7); COMPUTE_LOC(8);
-#endif
 
 		/* left channel */
-#ifdef __FAST_REVERB__
-		*srce++ +=RVbufL1[loc1]-RVbufL2[loc2]+RVbufL3[loc3]-RVbufL4[loc4];
-#else
 		*srce++ +=RVbufL1[loc1]-RVbufL2[loc2]+RVbufL3[loc3]-RVbufL4[loc4]+
 		          RVbufL5[loc5]-RVbufL6[loc6]+RVbufL7[loc7]-RVbufL8[loc8];
-#endif
 	}
 }
 
@@ -448,56 +425,38 @@ static void MixReverb_Stereo(SLONG* srce,NATIVE count)
 	unsigned int speedup;
 	int          ReverbPct;
 	unsigned int loc1, loc2, loc3, loc4;
-#ifndef __FAST_REVERB__
 	unsigned int loc5, loc6, loc7, loc8;
 
-	ReverbPct = 63 + (md_reverb*4);
-#else
-	ReverbPct = 92 + (md_reverb*2);
-#endif
+	ReverbPct = 92+(md_reverb<<1);
 
 	COMPUTE_LOC(1); COMPUTE_LOC(2); COMPUTE_LOC(3); COMPUTE_LOC(4);
-#ifndef __FAST_REVERB__
 	COMPUTE_LOC(5); COMPUTE_LOC(6); COMPUTE_LOC(7); COMPUTE_LOC(8);
-#endif
 
 	while(count--) {
 		/* Compute the left channel echo buffers */
 		speedup = *srce >> 3;
 
 		COMPUTE_LECHO(1); COMPUTE_LECHO(2); COMPUTE_LECHO(3); COMPUTE_LECHO(4);
-#ifndef __FAST_REVERB__
 		COMPUTE_LECHO(5); COMPUTE_LECHO(6); COMPUTE_LECHO(7); COMPUTE_LECHO(8);
-#endif
 
 		/* Compute the right channel echo buffers */
 		speedup = srce[1] >> 3;
 
 		COMPUTE_RECHO(1); COMPUTE_RECHO(2); COMPUTE_RECHO(3); COMPUTE_RECHO(4);
-#ifndef __FAST_REVERB__
 		COMPUTE_RECHO(5); COMPUTE_RECHO(6); COMPUTE_RECHO(7); COMPUTE_RECHO(8);
-#endif
 
 		/* Prepare to compute actual finalized data */
 		RVRindex++;
 
 		COMPUTE_LOC(1); COMPUTE_LOC(2); COMPUTE_LOC(3); COMPUTE_LOC(4);
-#ifndef __FAST_REVERB__
 		COMPUTE_LOC(5); COMPUTE_LOC(6); COMPUTE_LOC(7); COMPUTE_LOC(8);
-#endif
 
-#ifdef __FAST_REVERB__
-		/* left channel then right channel */
-		*srce++ +=RVbufL1[loc1]-RVbufL2[loc2]+RVbufL3[loc3]-RVbufL4[loc4];
-		*srce++ +=RVbufR1[loc1]-RVbufR2[loc2]+RVbufR3[loc3]-RVbufR4[loc4];
-#else
 		/* left channel then right channel */
 		*srce++ +=RVbufL1[loc1]-RVbufL2[loc2]+RVbufL3[loc3]-RVbufL4[loc4]+
 		          RVbufL5[loc5]-RVbufL6[loc6]+RVbufL7[loc7]-RVbufL8[loc8];
 
 		*srce++ +=RVbufR1[loc1]-RVbufR2[loc2]+RVbufR3[loc3]-RVbufR4[loc4]+
 		          RVbufR5[loc5]-RVbufR6[loc6]+RVbufR7[loc7]-RVbufR8[loc8];
-#endif
 	}
 }
 
@@ -719,7 +678,7 @@ void VC1_WriteSamples(SBYTE* buf,ULONG todo)
 				vnf = &vinf[t];
 
 				if(vnf->kick) {
-					vnf->current=vnf->start<<FRACBITS;
+					vnf->current=((SLONGLONG)vnf->start)<<FRACBITS;
 					vnf->kick   =0;
 					vnf->active =1;
 				}
@@ -741,9 +700,9 @@ void VC1_WriteSamples(SBYTE* buf,ULONG todo)
 					} else
 						vnf->lvolsel=vol;
 
-					idxsize = (vnf->size)? (vnf->size << FRACBITS)-1 : 0;
-					idxlend = (vnf->repend)? (vnf->repend << FRACBITS)-1 : 0;
-					idxlpos = vnf->reppos << FRACBITS;
+					idxsize = (vnf->size)? ((SLONGLONG)vnf->size << FRACBITS)-1 : 0;
+					idxlend = (vnf->repend)? ((SLONGLONG)vnf->repend << FRACBITS)-1 : 0;
+					idxlpos = (SLONGLONG)vnf->reppos << FRACBITS;
 					AddChannel(vc_tickbuf, portion);
 				}
 			}
@@ -766,6 +725,11 @@ void VC1_WriteSamples(SBYTE* buf,ULONG todo)
 
 BOOL VC1_Init(void)
 {
+	VC_SetupPointers();
+	
+	if (md_mode&DMODE_HQMIXER)
+		return VC2_Init();
+
 	if(!(Samples=(SWORD**)_mm_calloc(MAXSAMPLEHANDLES,sizeof(SWORD*)))) {
 		_mm_errno = MMERR_INITIALIZING_MIXER;
 		return 1;
@@ -791,34 +755,28 @@ BOOL VC1_PlayStart(void)
 	RVc2 = (5078L * md_mixfreq) / REVERBERATION;
 	RVc3 = (5313L * md_mixfreq) / REVERBERATION;
 	RVc4 = (5703L * md_mixfreq) / REVERBERATION;
-#ifndef __FAST_REVERB__
 	RVc5 = (6250L * md_mixfreq) / REVERBERATION;
 	RVc6 = (6953L * md_mixfreq) / REVERBERATION;
 	RVc7 = (7813L * md_mixfreq) / REVERBERATION;
 	RVc8 = (8828L * md_mixfreq) / REVERBERATION;
-#endif
 
 	if(!(RVbufL1=(SLONG*)_mm_calloc((RVc1+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufL2=(SLONG*)_mm_calloc((RVc2+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufL3=(SLONG*)_mm_calloc((RVc3+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufL4=(SLONG*)_mm_calloc((RVc4+1),sizeof(SLONG)))) return 1;
-#ifndef __FAST_REVERB__
 	if(!(RVbufL5=(SLONG*)_mm_calloc((RVc5+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufL6=(SLONG*)_mm_calloc((RVc6+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufL7=(SLONG*)_mm_calloc((RVc7+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufL8=(SLONG*)_mm_calloc((RVc8+1),sizeof(SLONG)))) return 1;
-#endif
 
 	if(!(RVbufR1=(SLONG*)_mm_calloc((RVc1+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufR2=(SLONG*)_mm_calloc((RVc2+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufR3=(SLONG*)_mm_calloc((RVc3+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufR4=(SLONG*)_mm_calloc((RVc4+1),sizeof(SLONG)))) return 1;
-#ifndef __FAST_REVERB__
 	if(!(RVbufR5=(SLONG*)_mm_calloc((RVc5+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufR6=(SLONG*)_mm_calloc((RVc6+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufR7=(SLONG*)_mm_calloc((RVc7+1),sizeof(SLONG)))) return 1;
 	if(!(RVbufR8=(SLONG*)_mm_calloc((RVc8+1),sizeof(SLONG)))) return 1;
-#endif
 
 	RVRindex = 0;
 	return 0;
@@ -830,26 +788,20 @@ void VC1_PlayStop(void)
 	if(RVbufL2) free(RVbufL2);
 	if(RVbufL3) free(RVbufL3);
 	if(RVbufL4) free(RVbufL4);
-	RVbufL1 = RVbufL2 = RVbufL3 = RVbufL4 = NULL;
-#ifndef __FAST_REVERB__
 	if(RVbufL5) free(RVbufL5);
 	if(RVbufL6) free(RVbufL6);
 	if(RVbufL7) free(RVbufL7);
 	if(RVbufL8) free(RVbufL8);
-	RVbufL5 = RVbufL6 = RVbufL7 = RVbufL8 = NULL;
-#endif
+	RVbufL1=RVbufL2=RVbufL3=RVbufL4=RVbufL5=RVbufL6=RVbufL7=RVbufL8=NULL;
 	if(RVbufR1) free(RVbufR1);
 	if(RVbufR2) free(RVbufR2);
 	if(RVbufR3) free(RVbufR3);
 	if(RVbufR4) free(RVbufR4);
-	RVbufR1 = RVbufR2 = RVbufR3 = RVbufR4 = NULL;
-#ifndef __FAST_REVERB__
 	if(RVbufR5) free(RVbufR5);
 	if(RVbufR6) free(RVbufR6);
 	if(RVbufR7) free(RVbufR7);
 	if(RVbufR8) free(RVbufR8);
-	RVbufR5 = RVbufR6 = RVbufR7 = RVbufR8 = NULL;
-#endif
+	RVbufR1=RVbufR2=RVbufR3=RVbufR4=RVbufR5=RVbufR6=RVbufR7=RVbufR8=NULL;
 }
 
 BOOL VC1_SetNumVoices(void)
@@ -869,14 +821,4 @@ BOOL VC1_SetNumVoices(void)
 	return 0;
 }
 
-void VC1_VoiceSetVolume(UBYTE voice,UWORD vol)
-{
-	vinf[voice].vol = vol;
-}
-
-void VC1_VoiceSetPanning(UBYTE voice, ULONG pan)
-{
-	vinf[voice].pan = pan;
-}
-
 /* ex:set ts=4: */
