@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: virtch2.c,v 1.22 1999/03/24 06:00:06 miod Exp $
+  $Id: virtch2.c,v 1.27 1999/06/28 03:00:06 miod Exp $
 
   High-quality sample mixing routines, using a 32 bits mixing buffer,
   interpolation, and sample smoothing to improve sound quality and remove
@@ -88,7 +88,7 @@
 #define TICKBSIZE (TICKWSIZE * 2)
 
 #define CLICK_SHIFT_BASE 6
-#define CLICK_SHIFT (CLICK_SHIFT_BASE * SAMPLING_SHIFT)
+#define CLICK_SHIFT (CLICK_SHIFT_BASE + SAMPLING_SHIFT)
 #define CLICK_BUFFER (1L << CLICK_SHIFT)
 
 #ifndef MIN
@@ -674,7 +674,9 @@ static void AddChannel(SLONG* ptr,NATIVE todo)
 #define VC1_WriteSamples      VC2_WriteSamples
 #define VC1_WriteBytes        VC2_WriteBytes
 #define VC1_Exit              VC2_Exit
+#define VC1_VoiceSetVolume    VC2_VoiceSetVolume
 #define VC1_VoiceGetVolume    VC2_VoiceGetVolume
+#define VC1_VoiceSetPanning   VC2_VoiceSetPanning
 #define VC1_VoiceGetPanning   VC2_VoiceGetPanning
 #define VC1_VoiceSetFrequency VC2_VoiceSetFrequency
 #define VC1_VoiceGetFrequency VC2_VoiceGetFrequency
@@ -718,7 +720,7 @@ void VC2_WriteSamples(SBYTE* buf,ULONG todo)
 				vnf = &vinf[t];
 
 				if(vnf->kick) {
-					vnf->current=(vnf->start) <<FRACBITS;
+					vnf->current=((SLONGLONG)(vnf->start))<<FRACBITS;
 					vnf->kick    = 0;
 					vnf->active  = 1;
 					vnf->click   = CLICK_BUFFER;
@@ -769,6 +771,11 @@ void VC2_WriteSamples(SBYTE* buf,ULONG todo)
 
 BOOL VC2_Init(void)
 {
+	VC_SetupPointers();
+	
+	if (!(md_mode&DMODE_HQMIXER))
+		return VC1_Init();
+	
 	if(!(Samples=(SWORD**)_mm_calloc(MAXSAMPLEHANDLES,sizeof(SWORD*)))) {
 		_mm_errno = MMERR_INITIALIZING_MIXER;
 		return 1;
@@ -874,20 +881,4 @@ BOOL VC2_SetNumVoices(void)
 	return 0;
 }
 
-void VC2_VoiceSetVolume(UBYTE voice,UWORD vol)
-{    
-	/* protect against clicks if volume variation is too high */
-	if(abs((int)vinf[voice].vol-(int)vol)>32)
-		vinf[voice].rampvol=CLICK_BUFFER;
-	vinf[voice].vol=vol;
-}
-
-void VC2_VoiceSetPanning(UBYTE voice,ULONG pan)
-{
-	/* protect against clicks if panning variation is too high */
-	if(abs((int)vinf[voice].pan-(int)pan)>48)
-		vinf[voice].rampvol=CLICK_BUFFER;
-	vinf[voice].pan=pan;
-}
-
 /* ex:set ts=4: */
